/*                                                                    */
/* wmab-0.1                                                           */
/*                                                                    */
/* A  WindowMaker dockable  application that  allows laptop  users to */
/* graphically  monitor  the  status  of their  power  source.  (I.e. */
/* whether or not AC or battery is in use as well as how long it will */
/* take  to  drain  or  charge the  battery.)  It  supports  multiple */
/* batteries simple with -b option.                                   */
/*                                                                    */
/* Written by Filippo Panessa  <kalem@linux.it> and copyrighted under */
/* GPL.                                                               */
/*                                                                    */
/* This  file  is  based  on wmacpi's idea  originally  written  (and */
/* copyrighted under GPL) by Timecop <timecop@japan.co.jp>.           */
/*                                                                    */
/*                                                                    */
/* This  program is  free software;  you can  redistribute it  and/or */
/* modify it  under the terms  of the  GNU General Public  License as */
/* published by  the Free Software  Foundation; either version  2, or */
/* (at your option) any later version.                                */
/*                                                                    */
/* This program  is distributed in the  hope that it will  be useful, */
/* but WITHOUT  ANY WARRANTY;  without even  the implied  warranty of */
/* MERCHANTABILITY or FITNESS  FOR A PARTICULAR PURPOSE.  See the GNU */
/* General Public License for more details.                           */
/*                                                                    */
/* You should have received a copy  of the GNU General Public License */
/* along with this  program (see the file COPYING); if  not, write to */
/* the Free Software  Foundation, Inc., 59 Temple Place  - Suite 330, */
/* Boston, MA 02111-1307, USA                                         */
/*                                                                    */
/*                                                                    */
/*                                                                    */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/types.h>
#include <dirent.h>

#include "libacpi.h"

/* see if we have ACPI support and check version */

int
check_acpi ()
{
  FILE *acpi;
  char buf[4096];
  DIR *battdir;
  struct dirent *batt;
  char *name;
  char *tmp;

  if (!(acpi = fopen ("/proc/acpi/info", "r")))
    {
      return 1;
    }

  /* okay, now see if we got the right version */

  fread (buf, 4096, 1, acpi);
  tmp = strstr (buf, "ersion:");
  if (tmp)
    {
      tmp += 16;
      if (atoi (tmp) < 20020208)
	{
	  fclose (acpi);
	  return 2;
	}
    }
  else
    {
      fclose (acpi);
      return 3;
    }

  /* yep, all good */
  fclose (acpi);

  /* now enumerate batteries */
  batt_count = 0;
  battdir = opendir ("/proc/acpi/battery");
  if (battdir == 0)
    {
      return 4;
    }
  while ((batt = readdir (battdir)))
    {
      name = batt->d_name;

      /* skip . and .. */
      if (!strncmp (".", name, 1) || !strncmp ("..", name, 2))
	continue;

      sprintf (batteries[batt_count], "/proc/acpi/battery/%s/state", name);
      sprintf (battinfo[batt_count], "/proc/acpi/battery/%s/info", name);
      batt_count++;
    }
  closedir (battdir);

  return 0;
}

int
read_acad_state ()
{
  FILE *acpi;
  char *ptr;
  char stat;

  if (!(acpi = fopen ("/proc/acpi/ac_adapter/ACAD/state", "r")))
    return -1;

  fread (buf, 512, 1, acpi);
  fclose (acpi);

  if ((ptr = strstr (buf, "state:")))
    {
      stat = *(ptr + 26);
      if (stat == 'n')
	{
	  acadstate->state = 1;
	}
      else
	{
	  acadstate->state = 0;
	  return 0;
	}
    }
  return 1;
}

int
read_acpi_info (int battery)
{
  FILE *acpi;
  char *ptr;
  char stat;
  int temp;

  if (battery > MAXBATT)
    return -1;

  if (!(acpi = fopen (battinfo[battery], "r")))
    return -1;

  fread (buf, 512, 1, acpi);
  fclose (acpi);

  if ((ptr = strstr (buf, "present:")))
    {
      stat = *(ptr + 25);
      if (stat == 'y')
	{
	  acpiinfo->present = 1;

	  if ((ptr = strstr (buf, "design capacity:")))
	    {
	      ptr += 25;
	      sscanf (ptr, "%d", &temp);
	      acpiinfo->design_capacity = temp;
	    }

	  if ((ptr = strstr (buf, "last full capacity:")))
	    {
	      ptr += 25;
	      sscanf (ptr, "%d", &temp);
	      acpiinfo->last_full_capacity = temp;
	    }

	  if ((ptr = strstr (buf, "battery technology:")))
	    {
	      stat = *(ptr + 25);
	      switch (stat)
		{
		case 'n':
		  acpiinfo->battery_technology = 1;
		  break;
		case 'r':
		  acpiinfo->battery_technology = 0;
		  break;
		}
	    }

	  if ((ptr = strstr (buf, "design voltage:")))
	    {
	      ptr += 25;
	      sscanf (ptr, "%d", &temp);
	      acpiinfo->design_voltage = temp;
	    }

	  if ((ptr = strstr (buf, "design capacity warning:")))
	    {
	      ptr += 25;
	      sscanf (ptr, "%d", &temp);
	      acpiinfo->design_capacity_warning = temp;
	    }

	  if ((ptr = strstr (buf, "design capacity low:")))
	    {
	      ptr += 25;
	      sscanf (ptr, "%d", &temp);
	      acpiinfo->design_capacity_low = temp;
	    }

	  if ((ptr = strstr (buf, "capacity granularity 1:")))
	    {
	      ptr += 25;
	      sscanf (ptr, "%d", &temp);
	      acpiinfo->capacity_granularity_1 = temp;
	    }

	  if ((ptr = strstr (buf, "capacity granularity 2:")))
	    {
	      ptr += 25;
	      sscanf (ptr, "%d", &temp);
	      acpiinfo->capacity_granularity_2 = temp;
	    }

	  if ((ptr = strstr (buf, "model number:")))
	    {
	      ptr += 25;
	      sscanf (ptr, "%s", &stat);
	      acpiinfo->model_number = stat;
	    }

	  if ((ptr = strstr (buf, "serial number:")))
	    {
	      ptr += 25;
	      sscanf (ptr, "%s", &stat);
	      acpiinfo->serial_number = stat;
	    }

	  if ((ptr = strstr (buf, "battery type:")))
	    {
	      ptr += 25;
	      sscanf (ptr, "%s", &stat);
	      acpiinfo->battery_type = stat;
	    }

	  if ((ptr = strstr (buf, "OEM info:")))
	    {
	      ptr += 25;
	      sscanf (ptr, "%s", &stat);
	      acpiinfo->OEM_info = stat;
	    }
	}
      else
	// Battery not present
	{
	  acpiinfo->present = 0;
	  acpiinfo->design_capacity = 0;
	  acpiinfo->last_full_capacity = 0;
	  acpiinfo->battery_technology = 0;
	  acpiinfo->design_voltage = 0;
	  acpiinfo->design_capacity_warning = 0;
	  acpiinfo->design_capacity_low = 0;
	  acpiinfo->capacity_granularity_1 = 0;
	  acpiinfo->capacity_granularity_2 = 0;
	  acpiinfo->model_number = 0;
	  acpiinfo->serial_number = 0;
	  acpiinfo->battery_type = 0;
	  acpiinfo->OEM_info = 0;
	  return 0;
	}
    }
  return 1;
}

int
read_acpi_state (int battery)
{
  FILE *acpi;
  char *ptr;
  char stat;

  int percent = 100;		/* battery percentage */
  int ptemp, rate, rtime = 0;

  if (!(acpi = fopen (batteries[battery], "r")))
    {
      return -1;
    }

  fread (buf, 512, 1, acpi);
  fclose (acpi);

  if ((ptr = strstr (buf, "present:")))
    {
      stat = *(ptr + 25);
      if (stat == 'y')
	{
	  acpistate->present = 1;

	  if ((ptr = strstr (buf, "capacity state:")))
	    {
	      stat = *(ptr + 25);
	      if (stat == 'o')
		// capacity state ok
		acpistate->capacity = 1;
	      else
		// capacity state critical
		acpistate->capacity = 0;
	    }

	  if ((ptr = strstr (buf, "charging state:")))
	    {
	      stat = *(ptr + 25);
	      switch (stat)
		{
		case 'd':
		  acpistate->state = 1;
		  break;
		case 'c':
		  if (*(ptr + 33) == '/')
		    acpistate->state = 0;
		  else
		    acpistate->state = 2;
		  break;
		case 'u':
		  acpistate->state = 3;
		  break;
		}
	    }

	  /* This section of the code will calculate "percentage remaining"
	   * using battery capacity, and the following formula 
	   * (acpi spec 3.9.2):
	   * 
	   * percentage = (current_capacity / last_full_capacity) * 100;
	   *
	   */

	  if ((ptr = strstr (buf, "remaining capacity:")))
	    {
	      ptr += 25;
	      sscanf (ptr, "%d", &ptemp);
	      acpistate->rcapacity = ptemp;

	      percent =
		(float) ((float) ptemp /
			 (float) acpiinfo->last_full_capacity) * 100;
	      acpistate->percentage = percent;
	    }


	  if ((ptr = strstr (buf, "present rate:")))
	    {
	      ptr += 25;
	      sscanf (ptr, "%d", &rate);

	      /* if something wrong */
	      if (rate <= 0)
		rate = 0;

	      acpistate->prate = rate;

	      /* time remaining in minutes */
	      rtime = ((float) ((float) acpistate->rcapacity /
		 (float) acpistate->prate)) * 3600;
	      if (rtime <= 0)
		rtime = 0;

	      acpistate->rtime = rtime;

	    }

	  if ((ptr = strstr (buf, "present voltage:")))
	    {
	      ptr += 25;
	      sscanf (ptr, "%d", &ptemp);
	      acpistate->pvoltage = ptemp;
	    }

	}
      else
	// Battery not present
	{
	  acpistate->present = 0;
	  acpistate->capacity = 0;
	  acpistate->state = UNKNOW;
	  acpistate->prate = 0;
	  acpistate->rcapacity = 0;
	  acpistate->pvoltage = 0;

	  acpistate->rtime = 0;
	  acpistate->percentage = 0;
	  return 0;
	}
    }
  return 1;
}

/*
  acpistate = calloc (1, sizeof (ACPIstate));
  acpiinfo = calloc (1, sizeof (ACPIinfo));
  acadstate = calloc (1, sizeof (ACADstate));
*/
