/*                                                                    */
/* wmab-0.1                                                           */
/*                                                                    */
/* A  WindowMaker dockable  application that  allows laptop  users to */
/* graphically  monitor  the  status  of their  power  source.  (I.e. */
/* whether or not AC or battery is in use as well as how long it will */
/* take  to  drain  or  charge the  battery.)  It  supports  multiple */
/* batteries simple with -b option.                                   */
/*                                                                    */
/* Written by Filippo Panessa  <kalem@linux.it> and copyrighted under */
/* GPL.                                                               */
/*                                                                    */
/* Based  on wmapm-3.1.0  originally written  (and copyrighted  under */
/* GPL)  by  Chris  D.  Faulhaber  <jedgar@fxp.org>  and  extensively */
/* modified by Michael G. Henderson <mghenderson@lanl.gov>.           */
/*                                                                    */
/*                                                                    */
/* This  program is  free software;  you can  redistribute it  and/or */
/* modify it  under the terms  of the  GNU General Public  License as */
/* published by  the Free Software  Foundation; either version  2, or */
/* (at your option) any later version.                                */
/*                                                                    */
/* This program  is distributed in the  hope that it will  be useful, */
/* but WITHOUT  ANY WARRANTY;  without even  the implied  warranty of */
/* MERCHANTABILITY or FITNESS  FOR A PARTICULAR PURPOSE.  See the GNU */
/* General Public License for more details.                           */
/*                                                                    */
/* You should have received a copy  of the GNU General Public License */
/* along with this  program (see the file COPYING); if  not, write to */
/* the Free Software  Foundation, Inc., 59 Temple Place  - Suite 330, */
/* Boston, MA 02111-1307, USA                                         */
/*                                                                    */
/*                                                                    */
/*                                                                    */

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <X11/X.h>
#include <X11/xpm.h>
#include "wmab.h"
#include "libacpi.h"
#include "../wmgeneral/wmgeneral.h"

#include "../pixmaps/wmab_master.xpm"
#include "../pixmaps/wmab_master_LowColor.xpm"
#include "../pixmaps/wmab_mask.xbm"

void ParseCMDLine (int argc, char *argv[]);
void pressEvent (XButtonEvent * xev);

int Battery = 1;		/* Use Battery 1 for default */
int CriticalLevel = 10;
int LowLevel = 40;
float BlinkRate = 3.0;		/* blinks per second */
float UpdateRate = 0.8;		/* Number of updates per second */
int Beep = 0;			/* Controls beeping when you 
				   get to CriticalLevel: 
				   Off by default */
int Volume = 50;		/* ring bell at 50% volume */
int UseLowColorPixmap = 0;	/* Use a lower number of colors 
				   for the poor saps on 
				   8-bit displays -- common
				   on laptops! */
int Toggle_info = 0;

/*  
 *   main  
 */
int
main (int argc, char *argv[])
{

  int hour, min, digit;
  XEvent event;
  int m, mMax, n, nMax, k, Toggle;
  long int r, rMax, s, sMax;

  int error;

  BlinkRate = 3.0;
  UpdateRate = 1.0 / 1.25;

  /*
   *  Parse any command line arguments.
   */
  ParseCMDLine (argc, argv);

  BlinkRate = (BlinkRate >= 0.0) ? BlinkRate : -1.0 * BlinkRate;
  UpdateRate = (UpdateRate >= 0.0) ? UpdateRate : -1.0 * UpdateRate;

  nMax = (int) (1.0e6 / (2.0 * UpdateRate * DELAY));
  mMax = (BlinkRate > 0.0) ? (int) (1.0e6 / (2.0 * BlinkRate * DELAY)) : nMax;

  acpistate = calloc (1, sizeof (ACPIstate));
  acpiinfo = calloc (1, sizeof (ACPIinfo));
  acadstate = calloc (1, sizeof (ACADstate));

  /*  
   *  Check for ACPI support 
   */

  error = check_acpi ();

  if (error == 1)
    {
      fprintf (stderr, "This system does not support ACPI\n");
      exit (1);
    }
  if (error == 2)
    {
      fprintf (stderr,
	       "This version requires ACPI subsystem version 20010313\n");
      exit (1);
    }
  if (error == 3)
    {
      fprintf (stderr,
	       "Perhaps old ACPI subsystem - consider upgrading to\n"
	       "at least version 20010313\n");
      exit (1);
    }
  if (error == 4)
    {
      fprintf (stderr, "No batteries or ACPI not supported\n");
      exit (1);
    }

  if (UseLowColorPixmap)
    openXwindow (argc, argv, wmab_master_LowColor, wmab_mask_bits,
		 wmab_mask_width, wmab_mask_height);
  else
    openXwindow (argc, argv, wmab_master, wmab_mask_bits, wmab_mask_width,
		 wmab_mask_height);

  /*
   *     Show battery number
   */

  copyXPMArea ((Battery) * 6 + 4, 69, 6, 7, 23, 51);

  /*
   *     Loop until we die...
   */

  n = 32000;
  m = 32000;
  r = rMax + 1;
  s = sMax + 1;
  while (1)
    {

      /*
       *  Only process acpi info only every nMax cycles of this
       *  loop. We run it faster to catch the xevents like button 
       *  presses and expose events, etc...
       *  
       *  DELAY is set at 0.00625 seconds, so process acpi info
       *  every 1.25 seconds...
       *
       */
      if (n > nMax)
	{

	  n = 0;

	  /* 
	   *   Check AC status.
	   */

	  read_acad_state ();

	  if (acadstate->state == 1)
	    /*
	     *   AC on-line. I.e. we are "plugged-in".
	     */
	    copyXPMArea (68, 6, 12, 7, 31, 35);
	  else
	    /*
	       *   AC off-line. I.e. we are using battery.
	     */
	    copyXPMArea (68, 20, 12, 7, 31, 35);

	  /* 
	   *    Paste up the default charge status and time
	   */
	  copyXPMArea (104, 6, 5, 7, 6, 7);
	  copyXPMArea (83, 93, 41, 9, 15, 7);

	  /*
	   *  Check to see if we are charging.
	   */

	  read_acpi_info (Battery - 1);
	  read_acpi_state (Battery - 1);

	  if (acpistate->state == CHARGING)
	    {

	      /*
	       *   Battery Status: Charging.
	       */
	      copyXPMArea (98, 6, 5, 7, 6, 7);
	      copyXPMArea (75, 81, 1, 2, 17, 9);
	      copyXPMArea (75, 81, 1, 2, 17, 12);

	    }

	  /* 
	   *  Paste up the "Time Left". This time means:
	   *  
	   *         If not charging: Time left before battery drains to 0% 
	   *         If charging:     Time left before battery gets to maximum
	   *
	   */

	  if ((acpistate->rtime >= 86400) || (acpistate->rtime < -1))
	    {

	      /*
	       *  If battery_time is too large, it likely means that there is
	       *  no charging or discharging going on. So just display a "null"
	       *  indicator (--:--). 
	       *
	       */
	      copyXPMArea (83, 106, 41, 9, 15, 7);

	    }
	  else if (acpistate->rtime >= 0)
	    {

	      hour = acpistate->rtime / 60;
	      
	      min = acpistate->rtime % 60;

	      copyXPMArea ((hour / 10) * 7 + 5, 93, 7, 9, 21, 7);
	      /* Show 10's (hour) */

	      copyXPMArea ((hour % 10) * 7 + 5, 93, 7, 9, 29, 7);
	      /* Show 1's (hour)  */

	      copyXPMArea (76, 93, 2, 9, 38, 7);
	      /* colon            */

	      copyXPMArea ((min / 10) * 7 + 5, 93, 7, 9, 42, 7);
	      /* Show 10's (min)  */

	      copyXPMArea ((min % 10) * 7 + 5, 93, 7, 9, 50, 7);
	      /* Show 1's (min)   */

	    }

	  /* 
	   *   Do Battery Percentage.
	   */
	  copyXPMArea (76, 81, 19, 7, 7, 34);	/* Show Default % */
	  copyXPMArea (66, 31, 49, 9, 7, 21);	/* Show Default Meter */

	  if (acpistate->percentage == 100)
	    {
	      copyXPMArea (15, 81, 1, 7, 7, 34);
	      /* If 100%, show 100% */

	      copyXPMArea (5, 81, 6, 7, 9, 34);
	      copyXPMArea (5, 81, 6, 7, 15, 34);
	      copyXPMArea (64, 81, 7, 7, 21, 34);	/* Show '%' */
	      copyXPMArea (66, 42, 49, 9, 7, 21);	/* Show Meter */
	    }
	  else
	    {

	      if (acpistate->percentage >= 10)
		copyXPMArea ((acpistate->percentage / 10) * 6 + 4, 81, 6, 7,
			     9, 34);
	      /* Show 10's */
	      copyXPMArea ((acpistate->percentage % 10) * 6 + 4, 81, 6, 7, 15, 34);	/* Show 1's */
	      copyXPMArea (64, 81, 7, 7, 21, 34);	/* Show '%' */

	      /*
	       *  Show Meter
	       */
	      k = acpistate->percentage * 49 / 100;
	      copyXPMArea (66, 42, k, 9, 7, 21);
	      if (k % 2)
		copyXPMArea (66 + k - 1, 52, 1, 9, 7 + k - 1, 21);
	      else
		copyXPMArea (66 + k, 52, 1, 9, 7 + k, 21);
	    }
	}
      else
	{
	  /*
	   *  Update the counter. 
	   */
	  ++n;
	}

      /*
       *  This controls battery LED
       */

      if (acpistate->present == 1)
	{

	  if (m > mMax)
	    {

	      m = 0;

	      if (acpistate->rcapacity <= acpiinfo->design_capacity_warning
		  || acpistate->percentage <= CriticalLevel)
		{

		  /*
		   *  Battery Status: Critical.
		   *  Blink the red led on/off...
		   */
		  if (Toggle || (BlinkRate == 0.0))
		    {
		      if (Beep)
			XBell (display, Volume);
		      Toggle = 0;
		      copyXPMArea (4, 105, 16, 10, 43, 34);
		    }
		  else
		    {
		      Toggle = 1;
		      copyXPMArea (58, 105, 16, 10, 43, 34);
		    }

		}
	      else if (acpistate->rcapacity == acpiinfo->design_capacity_low
		       || acpistate->percentage <= LowLevel)

		{

		  /*
		   *  Battery Status: Low.
		   */
		  copyXPMArea (22, 105, 16, 10, 43, 34);	/* turn ON  yellow */

		}
	      else if (acpistate->state == POWER
		       || acpistate->percentage > LowLevel)
		{

		  /*
		   *  Battery Status: High.
		   */
		  copyXPMArea (40, 105, 16, 10, 43, 34);	/* turn ON  green  */
	        }
		      
	    }
	  else
	    {

	      /*
	       *  Update the counter. 
	       */
	      ++m;
	    }

	}
      else
	{
	  copyXPMArea (95, 19, 16, 10, 43, 34);
	}

      /* 
       * Process any pending X events.
       */

      while (XPending (display))
	{
	  XNextEvent (display, &event);
	  switch (event.type)
	    {
	    case Expose:
	      RedrawWindow ();
	      break;
	    case ButtonPress:
	      pressEvent (&event.xbutton);
	      break;
	    case ButtonRelease:
	      break;
	    }
	}

      /* 
       *  Redraw and wait for next update 
       */
      RedrawWindow ();
      usleep (DELAY);
    }
}

void
pressEvent (XButtonEvent * xev)
{

  int x = xev->x;
  int y = xev->y;

  if (x >= 32 && y >= 49 && x <= 59 && y <= 59)
    {
      if (Toggle_info == 0)
	{
	  copyXPMArea (100, 80, 26, 9, 33, 50);
	  Toggle_info = 1;
	}
      else if (Toggle_info == 1)
	{
	  copyXPMArea (100, 68, 26, 9, 33, 50);
	  Toggle_info = 0;
	}

      RedrawWindow ();

      printf ("Not yet implemented\n");

      //usleep(2000000L);
    }

  return;

}

/* 
 *   ParseCMDLine()  
 */
void
ParseCMDLine (int argc, char *argv[])
{
  char *cmdline;
  int i, j;
  char puke[20];

  for (i = 1; i < argc; i++)
    {
      cmdline = argv[i];

      if (cmdline[0] == '-')
	{
	  switch (cmdline[1])
	    {
	    case 'd':
	      ++i;
	      break;
	    case 'b':
	      Battery = atof (argv[++i]);
	      if (Battery > 8)
		{
		  printf
		    ("Uh, have you got a laptop with more than 8 batteries ?\n");
		  exit (1);
		}
	      break;
	    case 'C':
	      CriticalLevel = atoi (argv[++i]);
	      break;
	    case 'L':
	      LowLevel = atoi (argv[++i]);
	      break;
	    case 'l':
	      UseLowColorPixmap = 1;
	      break;
	    case 'B':
	      Beep = 1;
	      Volume = atoi (argv[++i]);
	      break;
	    default:
	      printf ("\nwmab version: %s\n", WMAB_VERSION);
	      printf ("usage:\n");
	      printf ("\t-display <display>\tUse alternate display\n");
	      printf ("\t-l\t\t\tUse a low-color pixmap\n");
	      printf ("\t-b <num>\t\tDefine battery to monitor (max 8)\n");
	      printf
		("\t-L <LowLevel>\t\tDefine level at which yellow LED turns on\n");
	      printf
		("\t-C <CriticalLevel>\tDefine level at which red LED turns on\n");
	      printf
		("\t-B <Volume>\t\tBeep at Critical Level (-100%% to 100%%)\n");
	      printf ("\t-h\t\t\tDisplay help screen\n\n");
	      exit (1);
	    }
	}
    }
}
